package com.zanxiang.common.utils;

import lombok.extern.slf4j.Slf4j;
import org.springframework.util.Assert;

import java.io.UnsupportedEncodingException;
import java.net.InetSocketAddress;
import java.net.URLDecoder;
import java.net.URLEncoder;
import java.util.Map;

/**
 * @Author wcc
 * @Date 2020/12/28 11:36
 * @Version 1.0
 * @Description
 */
@Slf4j
public class URIUtil {

    /**
     * 填充 url的请求参数，如果 params的值为空，不会写在 url上
     *
     * @param url
     * @param params
     * @return
     */
    public static String fillUrlParams(String url, Map<String, String> params, boolean ignoreBlank) {
        if (params == null || params.isEmpty()) {
            return url;
        }
        boolean first = !url.contains("?");
        StringBuilder urlBuilder = new StringBuilder(url);
        for (Map.Entry<String, String> param : params.entrySet()) {
            if (ignoreBlank && StringUtils.isBlank(param.getValue())) {
                continue;
            }
            if (first) {
                urlBuilder.append("?");
                first = false;
            } else {
                urlBuilder.append("&");
            }
            urlBuilder.append(param.getKey()).append("=").append(StringUtils.isBlank(param.getValue()) ? "" : param.getValue());
        }
        url = urlBuilder.toString();
        return url;
    }

    /**
     * 对应 js中的 encodeURIComponent
     *
     * @param uri
     * @return
     */
    public static String encodeURIComponent(String uri) {
        return encodeURIComponent(uri, "UTF-8");
    }

    /**
     * 对应 js中的 encodeURIComponent
     *
     * @param uri
     * @return
     */
    public static String encodeURIComponent(String uri, String enc) {
        try {
            return URLEncoder.encode(uri, enc)
                    .replaceAll("\\+", "%20")
                    .replaceAll("\\!", "%21")
                    .replaceAll("\\'", "%27")
                    .replaceAll("\\(", "%28")
                    .replaceAll("\\)", "%29")
                    .replaceAll("\\~", "%7E");
        } catch (UnsupportedEncodingException e) {
            log.error(e.getMessage(), e);
            return null;
        }
    }

    /**
     * 对应 js的 decodeURIComponent
     *
     * @param uri
     * @return
     */
    public static String decodeURIComponent(String uri) {
        return decodeURIComponent(uri, "UTF-8");
    }

    /**
     * 对应 js的 decodeURIComponent
     *
     * @param uri
     * @return
     */
    public static String decodeURIComponent(String uri, String enc) {
        try {
            return URLDecoder.decode(uri.replaceAll("%20", "+")
                    .replaceAll("%21", "!")
                    .replaceAll("%27", "'")
                    .replaceAll("%28", "(")
                    .replaceAll("%29", ")")
                    .replaceAll("%7E", "~"), enc);
        } catch (UnsupportedEncodingException e) {
            log.error(e.getMessage(), e);
            return null;
        }
    }

    public static InetSocketAddress parseURI(String uri, int defaultPort) {
        Assert.notNull(uri, "uri must not be null");
        String host;
        String portString = null;

        int colonPos = uri.indexOf(':');
        if (colonPos >= 0 && uri.indexOf(':', colonPos + 1) == -1) {
            // Exactly 1 colon. Split into host:port.
            host = uri.substring(0, colonPos);
            portString = uri.substring(colonPos + 1);
        } else {
            // 0 or 2+ colons. Bare hostname or IPv6 literal.
            host = uri;
        }

        int port = defaultPort;
        if (StringUtils.isNotBlank(portString)) {
            // Try to parse the whole port string as a number.
            Assert.isTrue(!portString.startsWith("+"), String.format("Cannot parse port number: %s", uri));
            try {
                port = Integer.parseInt(portString);
            } catch (NumberFormatException e) {
                throw new IllegalArgumentException(String.format("Cannot parse port number: %s", uri));
            }

            Assert.isTrue(isValidPort(port), String.format("Port number out of range: %s", uri));
        }

        return InetSocketAddress.createUnresolved(host, port);
    }

    /**
     * @param port the port number
     * @return {@literal true} for valid port numbers.
     */
    private static boolean isValidPort(int port) {
        return port >= 0 && port <= 65535;
    }
}
