package com.zanxiang.manage.service.Impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.CollectionUtils;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.zanxiang.common.utils.StringUtils;
import com.zanxiang.common.utils.bean.BeanUtils;
import com.zanxiang.manage.domain.dto.*;
import com.zanxiang.manage.domain.params.GameUserListParam;
import com.zanxiang.manage.domain.vo.GameUserListVO;
import com.zanxiang.manage.domain.vo.GameUserVO;
import com.zanxiang.manage.service.*;
import com.zanxiang.mybatis.entity.GameUser;
import com.zanxiang.mybatis.mapper.GameUserMapper;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.time.LocalDateTime;
import java.time.LocalTime;
import java.util.*;

/**
 * @author : lingfeng
 * @time : 2022-07-04
 * @description : 玩家游戏
 */
@Slf4j
@Service
public class GameUserServiceImpl extends ServiceImpl<GameUserMapper, GameUser> implements GameUserService {

    @Autowired
    private UserService userService;

    @Autowired
    private GameService gameService;

    @Autowired
    private GameCategoryService gameCategoryService;

    @Autowired
    private PromoChannelService promoChannelService;

    @Autowired
    private CpService cpService;

    @Autowired
    private PromoAccountService promoAccountService;

    /**
     * 查询玩家游戏列表
     *
     * @param param : 玩家游戏列表参数
     * @return : 返回单页数据
     */
    @Override
    public IPage<GameUserListVO> gameUserList(GameUserListParam param) {
        //游戏条件处理
        Map<Long, GameDTO> gameMap = gameService.gameCondition(param.getCpId(), param.getGameId(), param.getGameCategoryId());
        //根据条件, 匹配不到游戏
        if (gameMap != null && gameMap.isEmpty()) {
            return new Page<>();
        }
        //渠道列表
        List<Long> agentIdList;
        if (param.getChannelId() != null) {
            agentIdList = Collections.singletonList(param.getChannelId());
        } else {
            agentIdList = promoChannelService.listByAccountOrPitcherId(param.getAccountId(), param.getPitcherId());
        }
        //根据条件, 匹配渠道
        if (agentIdList != null && agentIdList.isEmpty()) {
            return new Page<>();
        }
        //玩家条件处理
        Map<Long, UserDTO> userMap = userService.userCondition(param.getUserId(), agentIdList, param.getUserName(),
                param.getNickname(), null, param.getAdId());
        //根据条件, 匹配不到玩家
        if (userMap != null && userMap.isEmpty()) {
            return new Page<>();
        }
        //相关用户id
        Set<Long> userIdSet = userMap == null ? null : userMap.keySet();
        Map<Long, PromoChannelDTO> channelMap = promoChannelService.promoChannelMap();
        Map<Long, CpDTO> cpMap = cpService.cpMap();
        Map<Long, GameCategoryDTO> gameCategoryMap = gameCategoryService.gameCategoryMap();
        Map<Long, PromoAccountDTO> promoAccountMap = promoAccountService.promoAccountMap();
        //执行查询
        return page(param.toPage(), new QueryWrapper<GameUser>().lambda()
                .in(CollectionUtils.isNotEmpty(userIdSet), GameUser::getUserId, userIdSet)
                .in(gameMap != null, GameUser::getGameId, gameMap != null ? gameMap.keySet() : null)
                .ge(param.getBeginDate() != null, GameUser::getCreateTime, param.getBeginDate() == null ? null : LocalDateTime.of(param.getBeginDate(), LocalTime.MIN))
                .le(param.getEndDate() != null, GameUser::getCreateTime, param.getEndDate() == null ? null : LocalDateTime.of(param.getEndDate(), LocalTime.MAX))
                .ge(param.getRechargeBeginDate() != null, GameUser::getLastRechargeTime, param.getRechargeBeginDate() == null ? null : LocalDateTime.of(param.getRechargeBeginDate(), LocalTime.MIN))
                .le(param.getRechargeEndDate() != null, GameUser::getLastRechargeTime, param.getRechargeEndDate() == null ? null : LocalDateTime.of(param.getRechargeEndDate(), LocalTime.MAX))
                .eq(param.getCustomerId() != null, GameUser::getCustomerId, param.getCustomerId())
                .eq(param.getIsGs() != null, GameUser::getIsGs, param.getIsGs())
                .gt(Objects.equals(param.getIsRecharge(), Boolean.TRUE), GameUser::getRechargeCount, 0)
                .eq(Objects.equals(param.getIsRecharge(), Boolean.FALSE), GameUser::getRechargeCount, 0)
                .eq(param.getVipLevel() != null && param.getVipLevel() <= 10, GameUser::getRoleVipMax, param.getVipLevel())
                .gt(param.getVipLevel() != null && param.getVipLevel() > 10, GameUser::getRoleVipMax, param.getVipLevel())
                .orderByDesc(GameUser::getCreateTime)
        ).convert(u -> this.toVo(u, userMap, gameMap, promoAccountMap, channelMap, cpMap, gameCategoryMap));
    }

    /**
     * 游戏玩家信息转换对象
     *
     * @param gameUser : 游戏玩家信息
     * @param userMap  : 用户信息
     * @param gameMap  : 游戏信息
     * @return : 返回游戏玩家信息
     */
    private GameUserListVO toVo(GameUser gameUser, Map<Long, UserDTO> userMap, Map<Long, GameDTO> gameMap, Map<Long, PromoAccountDTO> promoAccountMap,
                                Map<Long, PromoChannelDTO> channelMap, Map<Long, CpDTO> cpMap, Map<Long, GameCategoryDTO> gameCategoryMap) {
        GameUserListVO gameUserListVO = BeanUtils.copy(gameUser, GameUserListVO.class);
        if (Objects.isNull(gameUserListVO)) {
            return null;
        }
        UserDTO userDTO = StringUtils.isEmpty(userMap) ? null : userMap.get(gameUserListVO.getUserId());
        if (userDTO == null) {
            userDTO = userService.getById(gameUserListVO.getUserId());
        }
        if (userDTO != null) {
            gameUserListVO.setNickname(userDTO.getNickname());
            gameUserListVO.setUsername(userDTO.getUsername());
            gameUserListVO.setAgentId(userDTO.getAgentId());
            gameUserListVO.setChannel(channelMap.get(userDTO.getAgentId()) == null ? null : channelMap.get(userDTO.getAgentId()).getChannel());
            gameUserListVO.setAdId(userDTO.getAdId());
        }
        GameDTO gameDTO = StringUtils.isEmpty(gameMap) ? null : gameMap.get(gameUserListVO.getGameId());
        if (gameDTO == null) {
            gameDTO = gameService.getById(gameUserListVO.getGameId());
        }
        if (gameDTO != null) {
            gameUserListVO.setCpId(gameDTO.getCpId());
            gameUserListVO.setGameName(gameDTO.getName());
            gameUserListVO.setGameCategoryId(gameDTO.getCategory());
            gameUserListVO.setCpName(cpMap.get(gameDTO.getCpId()) == null ? null : cpMap.get(gameDTO.getCpId()).getCpName());
            gameUserListVO.setGameCategoryName(gameCategoryMap.get(gameDTO.getCategory()) == null ? null : gameCategoryMap.get(gameDTO.getCategory()).getName());
        }
        PromoChannelDTO promoChannelDTO = channelMap.get(gameUserListVO.getAgentId());
        if (promoChannelDTO != null) {
            PromoAccountDTO promoAccountDTO = promoAccountMap.get(promoChannelDTO.getAccountId());
            if (promoAccountDTO != null) {
                gameUserListVO.setPitcherId(promoAccountDTO.getPitcherId());
                gameUserListVO.setPitcherName(promoAccountDTO.getPitcherName());
                gameUserListVO.setAccountId(promoAccountDTO.getId());
                gameUserListVO.setAccountName(promoAccountDTO.getName());
                gameUserListVO.setMediaId(promoAccountDTO.getMediaId());
                gameUserListVO.setMediaName(promoAccountDTO.getMediaName());
            }
        }
        return gameUserListVO;
    }

    /**
     * 获取指定用户对应vip等级最高的用户信息
     *
     * @param userId : 用户信息
     * @return : 返回玩家信息
     */
    @Override
    public GameUserVO getMaxVipGameUser(Long userId) {
        List<GameUser> gameUserList = super.list(new LambdaQueryWrapper<GameUser>()
                .eq(GameUser::getUserId, userId)
                .select(GameUser::getUserId, GameUser::getRoleVipMax)
                .orderByDesc(GameUser::getRoleVipMax));
        return BeanUtils.copy(gameUserList.get(0), GameUserVO.class);
    }

    /**
     * 获取指定用户对应vip等级最高的用户信息
     *
     * @param id : 主键id
     * @return : 返回玩家信息
     */
    @Override
    public GameUserVO getById(Long id) {
        return BeanUtils.copy(super.getById(id), GameUserVO.class);
    }
}
