package com.zanxiang.sdk.service.Impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.zanxiang.common.domain.ResultVO;
import com.zanxiang.common.enums.AccountStatusEnum;
import com.zanxiang.common.enums.HttpStatusEnum;
import com.zanxiang.common.enums.SmsTypeEnum;
import com.zanxiang.common.exception.BaseException;
import com.zanxiang.common.text.UUID;
import com.zanxiang.common.utils.URIUtil;
import com.zanxiang.module.util.JsonUtil;
import com.zanxiang.module.web.util.IpUtil;
import com.zanxiang.mybatis.entity.GameExt;
import com.zanxiang.mybatis.entity.User;
import com.zanxiang.mybatis.entity.UserCard;
import com.zanxiang.sdk.constant.ApiUrlConstant;
import com.zanxiang.sdk.constant.RedisKeyConstant;
import com.zanxiang.sdk.domain.dto.UserOauthDTO;
import com.zanxiang.sdk.domain.params.LoginMobileParam;
import com.zanxiang.sdk.domain.params.LoginPasswordParam;
import com.zanxiang.sdk.domain.params.QqLoginCallbackParam;
import com.zanxiang.sdk.domain.params.UserData;
import com.zanxiang.sdk.domain.vo.UserLoginVO;
import com.zanxiang.sdk.service.*;
import com.zanxiang.sdk.util.RedisUtil;
import com.zanxiang.sdk.util.RegexUtil;
import com.zanxiang.sdk.util.RegisterUtil;
import lombok.extern.slf4j.Slf4j;
import org.apache.logging.log4j.util.Strings;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.time.LocalDateTime;
import java.util.HashMap;
import java.util.Map;
import java.util.Objects;

/**
 * @author : lingfeng
 * @time : 2022-06-22
 * @description : 注册登录逻辑
 */
@Slf4j
@Service
public class LoginServiceImpl implements RegisterLoginService {

    @Autowired
    private RedisUtil<String> redisUtil;

    @Resource
    private QqApiService qqApiService;

    @Resource
    private VxApiService vxApiService;

    @Autowired
    private UserOauthService userOauthService;

    @Autowired
    private UserTokenService userTokenService;

    @Autowired
    private UserLoginLogService userLoginLogService;

    @Autowired
    private UserService userService;

    @Autowired
    private SmsService smsService;

    @Autowired
    private WordCheckService wordCheckService;

    @Autowired
    private PromoChannelService promoChannelService;

    @Autowired
    private UserCardService userCardService;

    @Autowired
    private GameUserService gameUserService;

    @Autowired
    private GameExtService gameExtService;

    @Autowired
    private IpBanService ipBanService;

    /**
     * QQ开发者应用id
     */
    @Value("${auth.qq-appId}")
    private String qqAppId;

    /**
     * 微信开发者应用id
     */
    @Value("${auth.vx-appId}")
    private String vxAppId;

    /**
     * 用户名注册或者登录
     *
     * @param param    : 注册登录参数
     * @param userData : 用户信息
     * @return : 返回用户登录信息
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public ResultVO<UserLoginVO> loginPassword(LoginPasswordParam param, UserData userData) {
        //验证登录ip是否封禁
        if (ipBanService.checkIpBan(userData.getIp())) {
            return new ResultVO<>(HttpStatusEnum.IP_HALT);
        }
        String username = param.getUsername();
        String password = param.getPassword();
        //用户信息
        User user;
        //登录, 进行登录检测
        if (Objects.equals(param.getType(), LoginPasswordParam.LOGIN)) {
            //判断是否手机号
            if (RegexUtil.checkPhone(username)) {
                user = userService.getOne(new LambdaQueryWrapper<User>().eq(User::getMobile, username));
            } else {
                //根据用户名查询
                user = userService.getOne(new LambdaQueryWrapper<User>().eq(User::getUsername, username));
            }
            //用户信息不存在
            if (user == null) {
                return new ResultVO<>(HttpStatusEnum.USERNAME_OR_PASSWORD_ERR);
            }
            //判断账号是否停用
            if (Objects.equals(AccountStatusEnum.FROZEN_STATUS.getStatus(), user.getStatus())) {
                return new ResultVO<>(HttpStatusEnum.ACCOUNT_HALT);
            }
            //验证密码
            if (!Objects.equals(RegisterUtil.cmfPassword(password), user.getPassword())) {
                return new ResultVO<>(HttpStatusEnum.USERNAME_OR_PASSWORD_ERR);
            }
        } else {
            //注册, 用户名密码校验
            HttpStatusEnum checkRegisterEnum = this.checkRegister(username, password);
            if (!Objects.equals(checkRegisterEnum, HttpStatusEnum.SUCCESS)) {
                return new ResultVO<>(checkRegisterEnum);
            }
            //创建用户信息
            user = User.builder()
                    .agentId(promoChannelService.getAgentIdByChannel(userData.getChannel()))
                    .gameId(userData.getGameId())
                    .username(username)
                    .nickname(RegisterUtil.randomNickName(username))
                    .password(RegisterUtil.cmfPassword(password))
                    .deviceType(userData.getDeviceType())
                    .status(AccountStatusEnum.NORMAL_STATUS.getStatus())
                    .authentication(0)
                    .createTime(LocalDateTime.now())
                    .updateTime(LocalDateTime.now())
                    .deviceSystem(userData.getDeviceSystem())
                    .mac(userData.getMac())
                    .imei(userData.getImei())
                    .androidId(userData.getAndroidId())
                    .ip(userData.getIp())
                    .build();
            userService.save(user);
            gameUserService.createGameUser(user);
        }
        //返回登录信息
        return new ResultVO<>(this.createUserLoginVO(user, userData));
    }

    /**
     * 手机号登录
     *
     * @param param    : 手机号登录参数
     * @param userData : 用户信息
     * @return : 返回登录信息
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public ResultVO<UserLoginVO> loginMobile(LoginMobileParam param, UserData userData) {
        //验证ip是否封禁
        if (ipBanService.checkIpBan(userData.getIp())) {
            return new ResultVO<>(HttpStatusEnum.IP_HALT);
        }
        String mobile = param.getMobile();
        //验证码校验
        HttpStatusEnum httpStatusEnum = smsService.smsCheck(SmsTypeEnum.SMS_REG.getType(), mobile, param.getCode());
        //验证不通过, 返回
        if (!Objects.equals(httpStatusEnum, HttpStatusEnum.SUCCESS)) {
            return new ResultVO<>(httpStatusEnum);
        }
        //获取用户信息
        User user = userService.getOne(new LambdaQueryWrapper<User>().eq(User::getMobile, mobile));
        //用户信息不存在, 直接注册
        if (user == null) {
            user = User.builder()
                    .agentId(promoChannelService.getAgentIdByChannel(userData.getChannel()))
                    .gameId(userData.getGameId())
                    .username(mobile)
                    .nickname(RegisterUtil.randomNickName(mobile))
                    .mobile(mobile)
                    .deviceType(userData.getDeviceType())
                    .status(AccountStatusEnum.NORMAL_STATUS.getStatus())
                    .authentication(0)
                    .createTime(LocalDateTime.now())
                    .updateTime(LocalDateTime.now())
                    .deviceSystem(userData.getDeviceSystem())
                    .mac(userData.getMac())
                    .imei(userData.getImei())
                    .androidId(userData.getAndroidId())
                    .ip(userData.getIp())
                    .build();
            userService.save(user);
            gameUserService.createGameUser(user);
        } else {
            //判断账号是否停用
            if (Objects.equals(AccountStatusEnum.FROZEN_STATUS.getStatus(), user.getStatus())) {
                return new ResultVO<>(HttpStatusEnum.ACCOUNT_HALT);
            }
        }
        //返回登录信息
        return new ResultVO<>(this.createUserLoginVO(user, userData));
    }

    /**
     * 构造用户登录信息
     *
     * @param user     : 用户
     * @param userData : 信息
     * @return : 返回结果
     */
    private UserLoginVO createUserLoginVO(User user, UserData userData) {
        //获取token
        String userToken = userTokenService.getUserToken(user.getId(), userData.getDeviceType());
        //插入用户登录记录
        userLoginLogService.addUserLoginLog(userData.getIp(), user, userData.getGameId(), 0);
        //查询用户实名信息
        UserCard userCard = userCardService.getOne(new LambdaQueryWrapper<UserCard>().eq(UserCard::getUserId, user.getId()));
        //查询游戏
        GameExt gameExt = gameExtService.getByGameId(userData.getGameId());
        //构造用户登录信息
        UserLoginVO userLoginVO = UserLoginVO.builder()
                .userId(user.getId())
                .userName(user.getUsername())
                .token(userToken)
                .authentication(user.getAuthentication())
                .bindPhone(Strings.isBlank(user.getMobile()) ? 0 : 1)
                .hasOldPassword(Strings.isNotBlank(user.getPassword()))
                .phoneNum(user.getShowPhoneNum())
                .cardName(userCard == null ? null : userCard.getShowCardName())
                .cardId(userCard == null ? null : userCard.getShowCardId())
                .checkSwitch(gameExt.getCheckSwitch())
                .build();
        log.error("用户登录返回登录信息, userLoginVO : {}", JsonUtil.toString(userLoginVO));
        return userLoginVO;
    }

    /**
     * 登录退出
     *
     * @param userData : 返回用户信息
     * @return : 返回结果
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public Boolean loginOut(UserData userData) {
        //新增用户退出记录
        return Boolean.TRUE;
    }

    /**
     * 登录注销/切换账号
     *
     * @param userData : 返回用户信息
     * @return : 返回结果
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public Boolean writeOff(UserData userData) {
        //删除token
        userTokenService.deleteUserToken(userData.getUserId(), userData.getDeviceType());
        return Boolean.TRUE;
    }

    /**
     * 登录用户名密码合规检测
     *
     * @param username : 用户名验证
     * @param password : 密码验证
     * @return : 返回验证结果
     */
    private HttpStatusEnum checkRegister(String username, String password) {
        //判断用户名是否存在敏感词
        if (wordCheckService.hasWord(username)) {
            return HttpStatusEnum.USERNAME_SENSITIVE;
        }
        //判断用户名是否已存在
        if (userService.count(new LambdaQueryWrapper<User>().eq(User::getUsername, username)) > 0) {
            return HttpStatusEnum.USERNAME_EXISTS;
        }
        //用户名合规检测
        HttpStatusEnum checkUserNameEnum = RegisterUtil.checkUserName(username);
        if (!Objects.equals(checkUserNameEnum, HttpStatusEnum.SUCCESS)) {
            return checkUserNameEnum;
        }
        //密码验证
        return RegisterUtil.checkPassword(password);
    }

    //-------------------------------- 微信和QQ授权登录(暂时没有用到) --------------------------------

    /**
     * QQ授权注册登录
     *
     * @param response : 返回体
     */
    @Override
    public void qqLoginAuth(HttpServletResponse response) {
        //获取state
        String state = this.getState();
        //传递参数
        Map<String, String> paramMap = new HashMap<>(4);
        paramMap.put("response_type", "code");
        paramMap.put("client_id", qqAppId);
        paramMap.put("state", state);
        paramMap.put("redirect_uri", ApiUrlConstant.QQ_REDIRECT_URL);
        //重定向到QQ授权页面
        try {
            response.sendRedirect(URIUtil.fillUrlParams(ApiUrlConstant.QQ_AUTH_URL, paramMap, Boolean.TRUE));
        } catch (Exception e) {
            log.error("QQ授权登录重定向跳转异常");
            throw new BaseException("QQ授权登录重定向跳转异常");
        }
    }

    /**
     * QQ授权注册登录回调
     *
     * @param param    : 回调参数
     * @param request  : request
     * @param userData : userData
     * @return : 返回登录token
     */
    @Override
    public ResultVO<UserLoginVO> qqLoginCallback(QqLoginCallbackParam param, HttpServletRequest request, UserData userData) {
        String code = param.getCode();
        String state = param.getState();
        Integer deviceType = userData.getDeviceType();
        //验证state，如果不一致，可能被CSRF攻击
        this.checkState(state);
        //获取用户信息
        Map<String, String> userInfoMap = qqApiService.qqAuthUserInfo(code);
        //查询用户授权信息是否存在
        UserOauthDTO userOauthDTO = userOauthService.getUserOauthByOpenId(userInfoMap.get("openId"));
        //获取用户信息
        User user = this.getUserByUserOauth(deviceType, userInfoMap, userOauthDTO);
        //用户登录成功
        String userToken = userTokenService.getUserToken(userOauthDTO.getId(), deviceType);
        //登录的ip
        String realIp = IpUtil.getRealIp(request);
        //插入用户登录记录
        userLoginLogService.addUserLoginLog(realIp, user, Long.valueOf(param.getGameId()), 0);
        //移出state
        redisUtil.removeOfSet(RedisKeyConstant.AUTH_STATE_KEY, state);
        //构造返回
        return new ResultVO<>(new UserLoginVO());
    }

    /**
     * 微信授权注册登录
     *
     * @param response : 返回体
     */
    @Override
    public void vxLoginAuth(HttpServletResponse response) {
        //获取state
        String state = this.getState();
        //传递参数
        Map<String, String> paramMap = new HashMap<>(6);
        paramMap.put("appid", qqAppId);
        paramMap.put("redirect_uri", URIUtil.encodeURIComponent(ApiUrlConstant.VX_REDIRECT_URL));
        paramMap.put("response_type", "code");
        paramMap.put("scope", "snsapi_login");
        paramMap.put("state", state);
        paramMap.put("lang", "cn");
        //重定向到QQ授权页面
        try {
            response.sendRedirect(URIUtil.fillUrlParams(ApiUrlConstant.VX_AUTH_URL, paramMap, Boolean.TRUE));
        } catch (Exception e) {
            log.error("微信授权登录重定向跳转异常");
            throw new BaseException("微信授权登录重定向跳转异常");
        }
    }

    /**
     * 微信授权注册登录回调
     *
     * @param param    : 回调参数
     * @param request  : request
     * @param userData : userData
     * @return : 返回登录token
     */
    @Override
    public ResultVO<UserLoginVO> vxLoginCallback(QqLoginCallbackParam param, HttpServletRequest request, UserData userData) {
        Integer deviceType = userData.getDeviceType();
        String code = param.getCode();
        String state = param.getState();
        //验证state，如果不一致，可能被CSRF攻击
        this.checkState(state);
        //获取用户信息
        Map<String, String> userInfoMap = vxApiService.vxAuthUserInfo(code);
        //查询用户授权信息是否存在
        UserOauthDTO userOauthDTO = userOauthService.getUserOauthByOpenId(userInfoMap.get("openId"));
        //获取用户信息
        User user = this.getUserByUserOauth(deviceType, userInfoMap, userOauthDTO);
        //登录的ip
        String realIp = IpUtil.getRealIp(request);
        //用户登录成功
        String userToken = userTokenService.getUserToken(userOauthDTO.getId(), deviceType);
        //插入用户登录记录
        userLoginLogService.addUserLoginLog(realIp, user, Long.valueOf(param.getGameId()), 0);
        //移出state
        redisUtil.removeOfSet(RedisKeyConstant.AUTH_STATE_KEY, state);
        //构造返回
        return new ResultVO<>(new UserLoginVO());
    }

    /**
     * 根据用户授权信息获取用户
     *
     * @param deviceType   : 设备类型
     * @param userInfoMap  : 第三方用户信息
     * @param userOauthDTO : 用户授权信息
     * @return : 返回应用用户信息
     */
    private User getUserByUserOauth(Integer deviceType, Map<String, String> userInfoMap, UserOauthDTO userOauthDTO) {
        if (userOauthDTO != null) {
            return userService.getById(userOauthDTO.getUserId());
        }
        //创建用户信息
        User user = User.builder()
                .nickname(userInfoMap.get("nickname"))
                .avatar(userInfoMap.get("avatar"))
                .deviceType(deviceType)
                .createTime(LocalDateTime.now())
                .updateTime(LocalDateTime.now())
                .build();
        userService.save(user);
        //创建用户授权信息
        userOauthDTO = userOauthService.createUserOauth(user, userInfoMap);
        log.info("用户授权信息, userOauthDTO : {}", JsonUtil.toString(userOauthDTO));
        //返回用户信息
        return user;
    }

    /**
     * state参数检测
     *
     * @param state : 随机密钥
     */
    private void checkState(String state) {
        if (!redisUtil.isMemberInSet(RedisKeyConstant.AUTH_STATE_KEY, state)) {
            throw new BaseException("State验证失败");
        }
    }

    /**
     * 生成随机密钥并添加到redis中
     *
     * @return : 返回随机密钥
     */
    private String getState() {
        String state = UUID.randomUUID().toString();
        redisUtil.addToSet(RedisKeyConstant.AUTH_STATE_KEY, state);
        return state;
    }
}
