package com.zanxiang.sdk.service.Impl;

import com.zanxiang.common.domain.ResultVO;
import com.zanxiang.common.enums.ExpireTimeEnum;
import com.zanxiang.common.enums.HttpStatusEnum;
import com.zanxiang.common.enums.SmsTypeEnum;
import com.zanxiang.module.sms.pojo.SendResult;
import com.zanxiang.module.sms.service.impl.AliSmsService;
import com.zanxiang.sdk.constant.RedisKeyConstant;
import com.zanxiang.sdk.domain.dto.UserDTO;
import com.zanxiang.sdk.domain.params.SmsCheckParam;
import com.zanxiang.sdk.domain.params.SmsSendParam;
import com.zanxiang.sdk.service.SmsService;
import com.zanxiang.sdk.service.UserService;
import com.zanxiang.sdk.util.RedisUtil;
import com.zanxiang.sdk.util.RegexUtil;
import lombok.extern.slf4j.Slf4j;
import org.apache.logging.log4j.util.Strings;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.Objects;
import java.util.Random;

/**
 * @author : lingfeng
 * @time : 2022-06-09
 * @description : 手机短信逻辑
 */
@Slf4j
@Service
public class SmsServiceImpl implements SmsService {

    @Autowired
    private UserService userService;

    @Autowired
    private RedisUtil<String> redisUtil;

    @Autowired
    private AliSmsService aliSmsService;

    /**
     * 发送短信验证码
     *
     * @param smsSendParam : 获取短信验证码参数
     * @return : 返回发送结果
     */
    @Override
    public ResultVO<Boolean> smsSend(SmsSendParam smsSendParam) {
        //类型
        Integer type = smsSendParam.getType();
        //手机号码
        String mobile = smsSendParam.getMobile();
        //根据手机号获取用户信息
        UserDTO userDTO = userService.getUserInfoByMobile(mobile);
        //找回密码获取
        if (Objects.equals(type, SmsTypeEnum.SMS_FIND_PWD.getType())) {
            //手机号对应用户信息不存在
            if (userDTO == null) {
                return new ResultVO<>(HttpStatusEnum.USERNAME_NOT_EXISTS);
            }
        }
        //验证手机号
        if (!RegexUtil.checkPhone(mobile)) {
            return new ResultVO<>(HttpStatusEnum.PHONE_ERROR);
        }
        //验证类型
        SmsTypeEnum smsTypeEnum = SmsTypeEnum.getByType(type);
        if (smsTypeEnum == null) {
            return new ResultVO<>(HttpStatusEnum.CODE_TYPE_ERROR);
        }
        //判断是否已经发送
        String cache = redisUtil.getCache(this.smsKey(mobile, type));
        if (Strings.isNotBlank(cache)) {
            return new ResultVO<>(HttpStatusEnum.PHONE_HAS_SEND);
        }
        //发送验证码
        String randomCode = this.randomCode();
        SendResult sendResult = aliSmsService.sendCode(randomCode, mobile);
        //发送成功设置缓存, 时效5分钟
        if (sendResult.isSuccess()) {
            redisUtil.setCache(this.smsKey(mobile, type), randomCode, ExpireTimeEnum.FIVE_MIN.getTime());
        }
        return new ResultVO<>(HttpStatusEnum.SUCCESS, Boolean.TRUE);
    }

    /**
     * 前端校验短信验证码
     *
     * @param smsCheckParam : 参数
     * @return : 返回验证结果和token
     */
    @Override
    public ResultVO<String> smsCheck(SmsCheckParam smsCheckParam) {
        //校验短信验证码
        HttpStatusEnum httpStatusEnum = this.smsCheck(smsCheckParam.getType(), smsCheckParam.getMobile(), smsCheckParam.getCode());
        //返回
        return new ResultVO<>(httpStatusEnum);
    }

    /**
     * 校验短信验证码
     *
     * @param type   : 短信类型
     * @param mobile : 手机号
     * @param code   : 验证码
     * @return : 返回验证结果
     */
    @Override
    public HttpStatusEnum smsCheck(Integer type, String mobile, String code) {
        //验证类型
        SmsTypeEnum smsTypeEnum = SmsTypeEnum.getByType(type);
        if (smsTypeEnum == null) {
            return HttpStatusEnum.CODE_TYPE_ERROR;
        }
        //验证手机号
        if (!RegexUtil.checkPhone(mobile)) {
            return HttpStatusEnum.PHONE_ERROR;
        }
        //获取缓存中的验证码
        String smsKey = this.smsKey(mobile, type);
        String cache = redisUtil.getCache(smsKey);
        //验证码已过期
        if (Strings.isBlank(cache)) {
            return HttpStatusEnum.CODE_IS_EXPIRED;
        }
        //验证码错误
        if (!Objects.equals(cache, code)) {
            return HttpStatusEnum.CODE_ERROR;
        }
        //验证成功, 删除缓存
        redisUtil.deleteCache(smsKey);
        //返回成功
        return HttpStatusEnum.SUCCESS;
    }

    /**
     * 获取手机验证码
     */
    private String randomCode() {
        Random random = new Random();
        return String.valueOf(random.nextInt(999999) + 100000);
    }

    /**
     * 手机验证码缓存key
     *
     * @param mobile : 手机号
     * @param type   : 短信类型
     * @return : 返回redis缓存key
     */
    private String smsKey(String mobile, Integer type) {
        return RedisKeyConstant.SMS_PHONE_KEY + "_" + mobile + "_" + type;
    }
}
