package com.zanxiang.sdk.service.Impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.LambdaUpdateWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.zanxiang.common.domain.ResultVO;
import com.zanxiang.common.enums.HttpStatusEnum;
import com.zanxiang.common.exception.BaseException;
import com.zanxiang.module.util.JsonUtil;
import com.zanxiang.mybatis.entity.User;
import com.zanxiang.mybatis.entity.UserCard;
import com.zanxiang.mybatis.mapper.UserCardMapper;
import com.zanxiang.sdk.constant.RegexConstant;
import com.zanxiang.sdk.domain.params.UserCardUpdateParam;
import com.zanxiang.sdk.domain.params.UserData;
import com.zanxiang.sdk.domain.result.CardCheckResult;
import com.zanxiang.sdk.service.UserCardService;
import com.zanxiang.sdk.service.UserService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.http.HttpEntity;
import org.springframework.http.HttpHeaders;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.client.RestTemplate;

import java.text.SimpleDateFormat;
import java.time.LocalDate;
import java.time.LocalDateTime;
import java.util.Calendar;
import java.util.Date;
import java.util.GregorianCalendar;
import java.util.UUID;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

/**
 * @author : lingfeng
 * @time : 2022-09-26
 * @description : 用户实名认证
 */
@Slf4j
@Service
public class UserCardServiceImpl extends ServiceImpl<UserCardMapper, UserCard> implements UserCardService {

    @Autowired
    private UserService userService;

    @Autowired
    private RestTemplate restTemplate;

    /**
     * 阿里实名应用code
     */
    @Value("${authentication.appCode}")
    private String appCode;

    /**
     * 用户实名认证
     *
     * @param param    : 实名认证参数
     * @param userData : 用户登录信息
     * @return : 返回结果
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public ResultVO<Boolean> userAuthentication(UserCardUpdateParam param, UserData userData) {
        //查询实名认证信息
        UserCard userCard = super.getOne(new LambdaQueryWrapper<UserCard>()
                .eq(UserCard::getUserId, userData.getUserId()));
        //已经实名认证, 不可重复认证
        if (userCard != null) {
            return new ResultVO<>(HttpStatusEnum.REPEAT_OPERATE);
        }
        //查询玩家信息
        User user = userService.getById(userData.getUserId());
        String cardId = param.getCardId();
        String cardName = param.getCardName();
        boolean nameCheck = this.cardNameCheck(cardName);
        //名字输入不合法
        if (!nameCheck) {
            return new ResultVO<>(HttpStatusEnum.CARD_NAME_ERROR);
        }
        boolean idCheck = this.cardIdCheck(cardId);
        //身份证号输入不合法
        if (!idCheck) {
            return new ResultVO<>(HttpStatusEnum.CARD_ID_ERROR);
        }
        //阿里实名认证
        boolean authenticationCheck = this.authenticationCheck(cardName, cardId);
        //实名认证失败
        if (!authenticationCheck) {
            return new ResultVO<>(HttpStatusEnum.AUTHENTICATION_FAIL);
        }
        //新建记录
        this.save(UserCard.builder()
                .userId(userData.getUserId())
                .username(user.getUsername())
                .nickname(user.getNickname())
                .regTime(user.getCreateTime())
                .cardId(cardId)
                .cardName(cardName)
                .birthday(this.getBirthday(cardId))
                .sex(this.getSex(cardId))
                .cardType(1)
                .createTime(LocalDateTime.now())
                .updateTime(LocalDateTime.now())
                .build());
        //更新用户信息
        userService.update(new LambdaUpdateWrapper<User>()
                .set(User::getAuthentication, this.getAge(cardId) >= 18 ? 1 : 2)
                .eq(User::getId, userData.getUserId()));
        return new ResultVO<>(HttpStatusEnum.SUCCESS, Boolean.TRUE);
    }

    /**
     * 姓名正则验证
     *
     * @param cardName : 姓名
     * @return : 返回是否合法
     */
    private boolean cardNameCheck(String cardName) {
        Pattern pattern = Pattern.compile(RegexConstant.REGEX_CHINESE_NAME);
        Matcher matcher = pattern.matcher(cardName);
        return matcher.matches();
    }

    /**
     * 身份证正则验证
     *
     * @param cardId : 身份证号
     * @return : 返回是否合法
     */
    private boolean cardIdCheck(String cardId) {
        String idValid = "(" + RegexConstant.REGEX_ID_CARD18 + ")" + "|" + "(" + RegexConstant.REGEX_ID_CARD15 + ")";
        Pattern pattern = Pattern.compile(idValid);
        Matcher matcher = pattern.matcher(cardId);
        return matcher.matches();
    }

    /**
     * 通过身份证号码获取生日
     *
     * @param cardId : 身份证号码
     * @return : 返回用户年龄
     */
    private String getBirthday(String cardId) {
        String birthday;
        if (cardId.length() == 15) {
            birthday = "19" + cardId.substring(6, 8) + "-" + cardId.substring(8, 10) + "-" + cardId.substring(10, 12);
        } else {
            birthday = cardId.substring(6, 10) + "-" + cardId.substring(10, 12) + "-" + cardId.substring(12, 14);
        }
        return birthday;
    }

    /**
     * 通过身份证号码获取年龄
     *
     * @param cardId : 身份证号码
     * @return : 返回用户年龄
     */
    private int getAge(String cardId) {
        String birthday;
        if (cardId.length() == 15) {
            birthday = "19" + cardId.substring(6, 8);
        } else {
            birthday = cardId.substring(6, 14);
        }
        Date birthDate = null;
        try {
            birthDate = new SimpleDateFormat("yyyyMMdd").parse(birthday);
        } catch (Exception e) {
            e.printStackTrace();
        }
        if (birthDate == null) {
            return 0;
        }
        GregorianCalendar currentDay = new GregorianCalendar();
        currentDay.setTime(birthDate);
        int birYear = currentDay.get(Calendar.YEAR);
        // 获取年龄
        return LocalDate.now().getYear() - birYear;
    }

    /**
     * 通过身份证号码获取性别
     *
     * @param cardId : 身份证号码
     * @return : 返回用户性别, 1:男，2:女
     */
    private int getSex(String cardId) {
        int sex;
        if (cardId.length() == 15) {
            sex = Integer.parseInt(cardId.substring(cardId.length() - 3)) % 2 == 0 ? 2 : 1;
        } else {
            sex = Integer.parseInt(cardId.substring(cardId.length() - 4, cardId.length() - 1)) % 2 == 0 ? 2 : 1;
        }
        return sex;
    }

    /**
     * 阿里实名认证
     *
     * @param cardName : 姓名
     * @param cardId   : 身份证号
     * @return : 返回是否匹配
     */
    private boolean authenticationCheck(String cardName, String cardId) {
        String host = "https://dskj.market.alicloudapi.com/platform/check/verified";
        String url = host + "?certCode=" + cardId + "&realName=" + cardName;
        HttpHeaders headers = new HttpHeaders();
        headers.set("Authorization", "APPCODE " + this.appCode);
        headers.set("X-Ca-Nonce", UUID.randomUUID().toString());
        HttpEntity httpEntity = new HttpEntity<>(headers);
        CardCheckResult result;
        try {
            String resultStr = restTemplate.postForObject(url, httpEntity, String.class);
            result = JsonUtil.toObj(resultStr, CardCheckResult.class);
        } catch (Exception e) {
            log.error("请求阿里实名认证接口异常, cardName : {}, cardId : {}, e : {}", cardName, cardId, e.getMessage());
            throw new BaseException("请求阿里实名认证接口异常");
        }
        if (result == null || !result.isSuccess()) {
            log.error("请求阿里实名认证接口返回值为空, cardName : {}, cardId : {}", cardName, cardId);
            throw new BaseException("实名认证失败");
        }
        return result.isPast();
    }
}
