package com.zanxiang.sdk.service.Impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.LambdaUpdateWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.zanxiang.common.domain.ResultVO;
import com.zanxiang.common.enums.HttpStatusEnum;
import com.zanxiang.common.enums.SmsTypeEnum;
import com.zanxiang.common.exception.BaseException;
import com.zanxiang.common.utils.bean.BeanUtils;
import com.zanxiang.module.util.JsonUtil;
import com.zanxiang.mybatis.entity.GameExt;
import com.zanxiang.mybatis.entity.User;
import com.zanxiang.mybatis.entity.UserCard;
import com.zanxiang.mybatis.mapper.UserMapper;
import com.zanxiang.sdk.domain.dto.UserDTO;
import com.zanxiang.sdk.domain.params.BindPhoneParam;
import com.zanxiang.sdk.domain.params.FindPasswordParam;
import com.zanxiang.sdk.domain.params.UpdatePasswordParam;
import com.zanxiang.sdk.domain.params.UserData;
import com.zanxiang.sdk.domain.vo.CustomerVO;
import com.zanxiang.sdk.domain.vo.UserVO;
import com.zanxiang.sdk.service.GameExtService;
import com.zanxiang.sdk.service.SmsService;
import com.zanxiang.sdk.service.UserCardService;
import com.zanxiang.sdk.service.UserService;
import com.zanxiang.sdk.util.RegisterUtil;
import lombok.extern.slf4j.Slf4j;
import org.apache.logging.log4j.util.Strings;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.time.LocalDateTime;
import java.util.Objects;

/**
 * @author : lingfeng
 * @time : 2022-06-07
 * @description : 用户信息
 */
@Slf4j
@Service
public class UserServiceImpl extends ServiceImpl<UserMapper, User> implements UserService {

    @Autowired
    private GameExtService gameExtService;

    @Autowired
    private SmsService smsService;

    @Autowired
    private UserCardService userCardService;

    /**
     * 忘记密码找回
     *
     * @param param : 修改密码的参数
     * @return : 返回修改结果
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public ResultVO<Boolean> findPassword(FindPasswordParam param) {
        String userName = param.getUserName();
        String mobile = param.getMobile();
        String code = param.getCode();
        String newPassword = param.getNewPassword();
        //查询用户信息
        User user;
        if (Strings.isNotBlank(userName)) {
            user = super.getOne(new LambdaQueryWrapper<User>()
                    .eq(User::getUsername, userName));
            if (user == null) {
                return new ResultVO<>(HttpStatusEnum.USERNAME_NOT_EXISTS);
            }
            if (Strings.isBlank(user.getMobile())) {
                return new ResultVO<>(HttpStatusEnum.ACCOUNT_NOT_BIND_PHONE);
            }
            if (!Objects.equals(mobile, user.getMobile())) {
                return new ResultVO<>(HttpStatusEnum.PHONE_NUM_ERROR);
            }
        } else {
            user = super.getOne(new LambdaQueryWrapper<User>()
                    .eq(User::getMobile, mobile));
            if (user == null) {
                return new ResultVO<>(HttpStatusEnum.PHONE_NOT_REG);
            }
        }
        //验证码校验
        HttpStatusEnum httpStatusEnum = smsService.smsCheck(SmsTypeEnum.SMS_FIND_PWD.getType(), mobile, code);
        //验证不通过, 返回
        if (!Objects.equals(httpStatusEnum, HttpStatusEnum.SUCCESS)) {
            return new ResultVO<>(httpStatusEnum);
        }
        //更新用户密码
        super.update(new LambdaUpdateWrapper<User>()
                .set(User::getUpdateTime, LocalDateTime.now())
                .set(User::getPassword, RegisterUtil.cmfPassword(newPassword))
                .eq(User::getId, user.getId())
        );
        //返回修改成功
        return new ResultVO<>(HttpStatusEnum.SUCCESS, Boolean.TRUE);
    }

    /**
     * 用户重置密码
     *
     * @param param    : 修改密码的参数
     * @param userData : userData
     * @return : 返回修改结果
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public ResultVO<Boolean> updatePassword(UpdatePasswordParam param, UserData userData) {
        String oldPassword = param.getOldPassword();
        String newPassword = param.getNewPassword();
        //查询用户信息
        User user = super.getById(userData.getUserId());
        //旧密存在
        if (Strings.isNotBlank(user.getPassword())) {
            //旧密码不可为空
            if (Strings.isBlank(oldPassword)) {
                return new ResultVO<>(HttpStatusEnum.OLD_PASSWORD_EMPTY);
            }
            //旧密码错误
            if (!Objects.equals(RegisterUtil.cmfPassword(oldPassword), user.getPassword())) {
                return new ResultVO<>(HttpStatusEnum.OLD_PASSWORD_ERROR);
            }
        }
        //新密码校验
        HttpStatusEnum checkPasswordEnum = RegisterUtil.checkPassword(newPassword);
        if (!Objects.equals(checkPasswordEnum, HttpStatusEnum.SUCCESS)) {
            return new ResultVO<>(checkPasswordEnum);
        }
        //修改用户密码
        super.update(new LambdaUpdateWrapper<User>()
                .set(User::getPassword, RegisterUtil.cmfPassword(newPassword))
                .set(User::getUpdateTime, LocalDateTime.now())
                .eq(User::getId, user.getId())
        );
        //返回修改成功
        return new ResultVO<>(HttpStatusEnum.SUCCESS, Boolean.TRUE);
    }

    /**
     * 用户绑定手机
     *
     * @param param    : 绑定手机参数
     * @param userData : userData
     * @return : 返回修改结果
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public ResultVO<Boolean> bindPhone(BindPhoneParam param, UserData userData) {
        String code = param.getCode();
        String mobile = param.getMobile();
        //判断是否已经绑定该账号
        User user = this.getById(userData.getUserId());
        if (user == null) {
            throw new BaseException("参数错误, 用户信息不存在");
        }
        //判断手机号是否已经绑定
        if (Strings.isNotBlank(user.getMobile()) && Objects.equals(user.getMobile(), mobile)) {
            return new ResultVO<>(HttpStatusEnum.PHONE_IS_BIND);
        }
        //校验手机号是否已经绑定了其他账号
        int count = super.count(new LambdaQueryWrapper<User>().eq(User::getMobile, mobile));
        if (count > 0) {
            return new ResultVO<>(HttpStatusEnum.PHONE_BIND_TOO_MUCH);
        }
        //验证码校验
        HttpStatusEnum httpStatusEnum = smsService.smsCheck(SmsTypeEnum.SMS_BIND.getType(), mobile, code);
        //验证不通过, 返回
        if (!Objects.equals(httpStatusEnum, HttpStatusEnum.SUCCESS)) {
            return new ResultVO<>(httpStatusEnum);
        }
        //修改用户用户手机号
        super.update(new LambdaUpdateWrapper<User>()
                .set(User::getMobile, mobile)
                .set(User::getUpdateTime, LocalDateTime.now())
                .eq(User::getId, userData.getUserId())
        );
        //返回修改成功
        return new ResultVO<>(HttpStatusEnum.SUCCESS, Boolean.TRUE);
    }

    /**
     * 获取用户信息
     *
     * @param userData : 用户信息
     * @return : 返回用户详情信息
     */
    @Override
    public UserVO getUserDetail(UserData userData) {
        Long userId = userData.getUserId();
        //查询用户信息
        User user = this.getById(userId);
        //查询用户实名信息
        UserCard userCard = userCardService.getOne(new LambdaQueryWrapper<UserCard>().eq(UserCard::getUserId, userId));
        //查询游戏
        GameExt gameExt = gameExtService.getByGameId(userData.getGameId());
        //构造用户登录信息
        UserVO userVO = UserVO.builder()
                .userId(user.getId())
                .userName(user.getUsername())
                .authentication(user.getAuthentication())
                .bindPhone(Strings.isBlank(user.getMobile()) ? 0 : 1)
                .hasOldPassword(Strings.isNotBlank(user.getPassword()))
                .phoneNum(user.getShowPhoneNum())
                .cardName(userCard == null ? null : userCard.getShowCardName())
                .cardId(userCard == null ? null : userCard.getShowCardId())
                .checkSwitch(gameExt.getCheckSwitch())
                .build();
        log.error("获取用户详情返回用户信息, userVO : {}", JsonUtil.toString(userVO));
        return userVO;
    }

    /**
     * 用户获取游戏客服信息
     *
     * @param userData : userData
     * @return : 返回用户信息
     */
    @Override
    public CustomerVO getUserCustomer(UserData userData) {
        Long gameId = userData.getGameId();
        GameExt gameExt = gameExtService.getByGameId(gameId);
        return CustomerVO.builder()
                .customerPhone(gameExt.getCustomerPhone())
                .customerQq(gameExt.getCustomerQq())
                .mpAppId(gameExt.getMpAppId())
                .mpAppName(gameExt.getMpAppName())
                .mpCode(gameExt.getMpCode())
                .build();
    }

    /**
     * 根据手机号获取用户信息
     *
     * @param mobile : 用户手机号
     * @return : 返回用户信息
     */
    @Override
    public UserDTO getUserInfoByMobile(String mobile) {
        User user = super.getOne(new LambdaQueryWrapper<User>().eq(User::getMobile, mobile));
        return BeanUtils.copy(user, UserDTO.class);
    }
}
