package com.zanxiang.sdk.service.Impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.zanxiang.common.constant.Constants;
import com.zanxiang.common.domain.ResultVO;
import com.zanxiang.common.enums.AccountStatusEnum;
import com.zanxiang.common.enums.ExpireTimeEnum;
import com.zanxiang.common.exception.BaseException;
import com.zanxiang.common.text.UUID;
import com.zanxiang.common.utils.DateUtils;
import com.zanxiang.common.utils.MD5Util;
import com.zanxiang.common.utils.bean.BeanUtils;
import com.zanxiang.mybatis.entity.GameExt;
import com.zanxiang.mybatis.entity.User;
import com.zanxiang.mybatis.entity.UserToken;
import com.zanxiang.mybatis.mapper.UserTokenMapper;
import com.zanxiang.sdk.constant.RedisKeyConstant;
import com.zanxiang.sdk.domain.dto.UserTokenDTO;
import com.zanxiang.sdk.domain.params.UserData;
import com.zanxiang.sdk.enums.TokenCheckEnum;
import com.zanxiang.sdk.service.GameExtService;
import com.zanxiang.sdk.service.IpBanService;
import com.zanxiang.sdk.service.UserService;
import com.zanxiang.sdk.service.UserTokenService;
import com.zanxiang.sdk.util.RedisUtil;
import com.zanxiang.sdk.util.SignUtil;
import lombok.extern.slf4j.Slf4j;
import org.apache.logging.log4j.util.Strings;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.time.LocalDateTime;
import java.util.Objects;

/**
 * @author : lingfeng
 * @time : 2022-06-07
 * @description : 用户token管理
 */
@Slf4j
@Service
public class UserTokenServiceImpl extends ServiceImpl<UserTokenMapper, UserToken> implements UserTokenService {

    @Autowired
    private RedisUtil<UserToken> redisUtil;

    @Autowired
    private GameExtService gameExtService;

    @Autowired
    private IpBanService ipBanService;

    @Autowired
    private UserService userService;

    /**
     * 登录凭证验证
     *
     * @param appId  : 游戏id
     * @param userId : 用户id
     * @param token  : 用户token
     * @param sign   : 签名
     * @return : 返回验证结果
     */
    @Override
    public ResultVO<Long> userTokenCheck(String appId, Long userId, String token, String sign) {
        //查询登录签名
        GameExt gameExt = gameExtService.getByGameAppId(appId);
        if (gameExt == null || Strings.isBlank(gameExt.getLoginKey())) {
            return ResultVO.ok(TokenCheckEnum.PARAM_LACK.getCode(), TokenCheckEnum.PARAM_LACK.getMsg());
        }
        //查询token是否存在
        UserToken userToken = super.getOne(new LambdaQueryWrapper<UserToken>()
                .eq(UserToken::getToken, token)
                .eq(UserToken::getUserId, userId));
        //判断token是否存在, 并且没有过期
        if (userToken == null || userToken.getExpireTime() < DateUtils.localDateTimeToSecond(LocalDateTime.now())) {
            return ResultVO.ok(TokenCheckEnum.SIGN_ERROR.getCode(), TokenCheckEnum.SIGN_ERROR.getMsg());
        }
        //登录密钥
        String loginKey = gameExt.getLoginKey();
        //计算用户签名
        StringBuilder sb = new StringBuilder();
        sb.append("loginKey=").append(loginKey);
        sb.append("&appId=").append(appId);
        sb.append("&userId=").append(userId);
        sb.append("&token=").append(token);
        String mySign;
        try {
            mySign = SignUtil.MD5(sb.toString()).toUpperCase();
        } catch (Exception e) {
            log.error("md5工具类加密异常, str : {}, e : {}", sb.toString(), e.getMessage());
            throw new BaseException("MD5加密异常");
        }
        //签名错误
        if (!Objects.equals(mySign, sign)) {
            return ResultVO.ok(TokenCheckEnum.CHECK_FAIL.getCode(), TokenCheckEnum.CHECK_FAIL.getMsg());
        }
        return ResultVO.ok(userId);
    }

    /**
     * 登录凭证验证
     *
     * @param userData : 用户信息
     * @return : 返回凭证是否过期
     */
    @Override
    public Boolean userTokenExpireTimeCheck(UserData userData) {
        //验证ip是否封禁
        if (ipBanService.checkIpBan(userData.getIp())) {
            return Boolean.FALSE;
        }
        //查询用户信息
        User user = userService.getById(userData.getUserId());
        //判断账号是否停用
        if (Objects.equals(AccountStatusEnum.FROZEN_STATUS.getStatus(), user.getStatus())) {
            return Boolean.FALSE;
        }
        //token检测
        Long expireTime = userData.getExpireTime();
        if (expireTime == null) {
            return Boolean.FALSE;
        }
        //判断token是否过期
        long endTime = DateUtils.localDateTimeToSecond(LocalDateTime.now().plusDays(1));
        //判断有效期时间是否大于1天
        return expireTime > endTime;
    }

    /**
     * 通过token设备获取token详情
     *
     * @param token      : 用户token
     * @param deviceType : 设备类型
     * @return : 返回用户token详情
     */
    @Override
    public UserTokenDTO getTokenInfoByTokenDevice(String token, Integer deviceType) {
        //查询缓存信息
        String tokenInfoKey = this.getTokenInfoKey(token, deviceType);
        UserToken userToken = redisUtil.getCache(tokenInfoKey);
        if (userToken != null) {
            return BeanUtils.copy(userToken, UserTokenDTO.class);
        }
        //缓存不存在, 查询数据库
        userToken = super.getOne(new LambdaQueryWrapper<UserToken>()
                .eq(UserToken::getToken, token)
                .eq(UserToken::getDeviceType, deviceType));
        if (userToken == null) {
            return null;
        }
        //设置token详情缓存
        redisUtil.setCache(tokenInfoKey, userToken, ExpireTimeEnum.ONE_DAY.getTime());
        //设置用户缓存
        String userTokenKey = this.getUserTokenKey(userToken.getUserId(), deviceType);
        redisUtil.setCache(userTokenKey, userToken, ExpireTimeEnum.ONE_DAY.getTime());
        //返回数据
        return BeanUtils.copy(userToken, UserTokenDTO.class);
    }

    /**
     * 获取用户token
     *
     * @param userId     : 用户id
     * @param deviceType : 设备类型
     * @return : 返回token
     */
    @Override
    public String getUserToken(Long userId, Integer deviceType) {
        //获取token
        String token = this.createToken(userId, deviceType);
        //查询数据库token信息
        UserToken userToken = super.getOne(new LambdaQueryWrapper<UserToken>()
                .eq(UserToken::getUserId, userId)
                .eq(UserToken::getDeviceType, deviceType));
        //不存在token数据, 直接创建
        if (userToken == null) {
            userToken = UserToken.builder()
                    .userId(userId)
                    .token(token)
                    .deviceType(deviceType)
                    .createTime(LocalDateTime.now())
                    .updateTime(LocalDateTime.now())
                    .expireTime(DateUtils.localDateTimeToSecond(LocalDateTime.now()) + ExpireTimeEnum.ONE_WEEK.getTime())
                    .build();
            super.save(userToken);
        } else {
            //更新数据库
            userToken.setToken(token);
            userToken.setUpdateTime(LocalDateTime.now());
            userToken.setExpireTime(DateUtils.localDateTimeToSecond(LocalDateTime.now()) + ExpireTimeEnum.ONE_WEEK.getTime());
            super.updateById(userToken);
        }
        //设置用户缓存
        String userTokenKey = this.getUserTokenKey(userId, deviceType);
        redisUtil.setCache(userTokenKey, userToken, ExpireTimeEnum.ONE_DAY.getTime());
        //设置token详情缓存
        String tokenInfoKey = this.getTokenInfoKey(token, deviceType);
        redisUtil.setCache(tokenInfoKey, userToken, ExpireTimeEnum.ONE_DAY.getTime());
        //返回token
        return token;
    }

    /**
     * 删除用户token
     *
     * @param userId     : 用户id
     * @param deviceType : 设备类型
     */
    @Override
    public void deleteUserToken(Long userId, Integer deviceType) {
        UserToken userToken = super.getOne(new LambdaQueryWrapper<UserToken>()
                .eq(UserToken::getUserId, userId)
                .eq(UserToken::getDeviceType, deviceType)
        );
        if (userToken == null) {
            return;
        }
        //根据主键删除数据库
        super.removeById(userToken.getId());
        //删除缓存
        String userTokenKey = this.getUserTokenKey(userId, userToken.getDeviceType());
        redisUtil.deleteCache(userTokenKey);
        String tokenInfoKey = this.getTokenInfoKey(userToken.getToken(), userToken.getDeviceType());
        redisUtil.deleteCache(tokenInfoKey);
    }

    /**
     * 创建token
     *
     * @param userId     : 用户id
     * @param deviceType : 设备类型
     * @return : 返回token
     */
    private String createToken(Long userId, Integer deviceType) {
        //生成随机数
        String random = userId + deviceType + UUID.randomUUID().toString() + System.currentTimeMillis();
        String charsetName = Constants.UTF8;
        //创建token
        return MD5Util.MD5Encode(MD5Util.MD5Encode(random, charsetName), charsetName);
    }

    /**
     * 拼接用户token缓存key
     *
     * @param userId     : 用户id
     * @param deviceType : 设备类型
     * @return : 返回用户token的缓存key
     */
    private String getUserTokenKey(Long userId, Integer deviceType) {
        return RedisKeyConstant.USER_TOKEN_KEY + "_" + userId + "_" + deviceType;
    }

    /**
     * 拼接用户token缓存key
     *
     * @param token      : 用户token
     * @param deviceType : 设备类型
     * @return : 返回用户token的缓存key
     */
    private String getTokenInfoKey(String token, Integer deviceType) {
        return RedisKeyConstant.TOKEN_INFO_KEY + "_" + token + "_" + deviceType;
    }
}
