package com.zanxiang.sdk.service.auth;

import cn.hutool.http.HttpUtil;
import com.zanxiang.common.exception.BaseException;
import com.zanxiang.common.utils.URIUtil;
import com.zanxiang.module.util.JsonUtil;
import com.zanxiang.sdk.constant.ApiUrlConstant;
import com.zanxiang.sdk.service.QqApiService;
import lombok.extern.slf4j.Slf4j;
import org.apache.logging.log4j.util.Strings;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import java.util.HashMap;
import java.util.Map;

/**
 * @author : lingfeng
 * @time : 2022-06-20
 * @description : QQ Api接口
 */
@Slf4j
@Service
public class QqApiServiceImpl implements QqApiService {

    /**
     * QQ开发者应用id
     */
    @Value("${auth.qq-appId}")
    private String qqAppId;

    /**
     * QQ开发者应用key
     */
    @Value("${auth.qq-appKey}")
    private String qqAppKey;

    /**
     * QQ授权获取用户信息
     *
     * @param code : 前端授权code
     * @return : 返回用户信息
     */
    @Override
    public Map<String, String> qqAuthUserInfo(String code) {
        // 向QQ认证服务器申请令牌
        String accessToken = this.getAccessTokenApi(code);
        if (Strings.isBlank(accessToken)) {
            throw new BaseException("获取token失败");
        }
        // 通过accessToken获取用户openId
        String openId = this.getUserOpenIdApi(accessToken);
        if (Strings.isBlank(openId)) {
            throw new BaseException("获取openId失败");
        }
        // 获取用户信息
        Map<String, String> userInfo = this.getUserInfoApi(accessToken, openId);
        if (userInfo.isEmpty()) {
            throw new BaseException("获取用户信息失败");
        }
        //构造返回信息
        Map<String, String> map = new HashMap<>(3);
        map.put("openId", openId);
        map.put("accessToken", accessToken);
        map.put("nickname", userInfo.get("nickname"));
        map.put("avatar", userInfo.get("figureurl_qq_1"));
        map.put("six", userInfo.get("gender"));
        return map;
    }

    /**
     * 获取用户授权token
     *
     * @param code : 用户授权code
     * @return : 返回token
     */
    private String getAccessTokenApi(String code) {
        Map<String, String> paramMap = new HashMap<>(5);
        paramMap.put("grant_type", "authorization_code");
        paramMap.put("code", code);
        paramMap.put("redirect_uri", ApiUrlConstant.QQ_REDIRECT_URL);
        paramMap.put("client_id", qqAppId);
        paramMap.put("client_secret", qqAppKey);
        //拼接url请求参数
        Map<String, String> resultMap = new HashMap<>();
        try {
            String uri = URIUtil.fillUrlParams(ApiUrlConstant.QQ_WEB_TOKEN_URL, paramMap, Boolean.TRUE);
            String result = HttpUtil.get(uri);
            if (Strings.isNotBlank(result)) {
                resultMap = JsonUtil.toMap(result, Map.class, String.class);
            }
        } catch (Exception e) {
            log.error("QQ获取用户授权token异常");
            throw new BaseException("QQ获取用户授权token异常");
        }
        return resultMap.get("access_token");
    }

    /**
     * 获取用户openId
     *
     * @param accessToken : 用户token密钥
     * @return : 返回用户openId
     */
    private String getUserOpenIdApi(String accessToken) {
        //拼接url请求参数
        Map<String, String> paramMap = new HashMap<>(1);
        paramMap.put("access_token", accessToken);
        Map<String, String> resultMap = new HashMap<>();
        try {
            String uri = URIUtil.fillUrlParams(ApiUrlConstant.QQ_USER_OPENID_URL, paramMap, Boolean.TRUE);
            String result = HttpUtil.get(uri);
            if (Strings.isNotBlank(result)) {
                resultMap = JsonUtil.toMap(result, Map.class, String.class);
            }
        } catch (Exception e) {
            log.error("");
        }
        return resultMap.get("openid");
    }

    /**
     * 获取用户openId
     *
     * @param accessToken : 用户token密钥
     * @return : 返回用户openId
     */
    private Map<String, String> getUserInfoApi(String accessToken, String openId) {
        //拼接url请求参数
        Map<String, String> paramMap = new HashMap<>(3);
        paramMap.put("access_token", accessToken);
        paramMap.put("oauth_consumer_key", qqAppId);
        paramMap.put("openid", openId);
        Map<String, String> resultMap = new HashMap<>();
        try {
            String uri = URIUtil.fillUrlParams(ApiUrlConstant.QQ_USER_INFO_URL, paramMap, Boolean.TRUE);
            String result = HttpUtil.get(uri);
            if (Strings.isNotBlank(result)) {
                resultMap = JsonUtil.toMap(result, Map.class, String.class);
            }
        } catch (Exception e) {
            log.error("");
        }
        return resultMap;
    }
}
