package com.zanxiang.sdk.service.pay;

import com.alibaba.fastjson.JSONObject;
import com.alipay.easysdk.factory.Factory;
import com.alipay.easysdk.kernel.Config;
import com.alipay.easysdk.kernel.util.ResponseChecker;
import com.alipay.easysdk.payment.app.models.AlipayTradeAppPayResponse;
import com.alipay.easysdk.payment.page.models.AlipayTradePagePayResponse;
import com.alipay.easysdk.payment.wap.models.AlipayTradeWapPayResponse;
import com.zanxiang.common.constant.Constants;
import com.zanxiang.common.domain.ResultMap;
import com.zanxiang.common.enums.HttpStatusEnum;
import com.zanxiang.common.enums.OsEnum;
import com.zanxiang.common.enums.ResEnum;
import com.zanxiang.module.util.JsonUtil;
import com.zanxiang.sdk.domain.bo.ProductPayParamBO;
import com.zanxiang.sdk.util.HttpUtil;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.net.URLEncoder;
import java.nio.charset.StandardCharsets;
import java.util.Enumeration;
import java.util.HashMap;
import java.util.Map;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

/**
 * @author xufeng
 * @date 2022/6/8 15:37
 */
@Slf4j
@Service
public class AliPayService extends PayBaseService {

    private Pattern pattern = Pattern.compile("action=\"(.*)\">", Pattern.CASE_INSENSITIVE);

    private Pattern param = Pattern.compile("value=\"(.*)\">", Pattern.CASE_INSENSITIVE);

    private Pattern mobileClientParam = Pattern.compile("\\{\"queryResultUrl\".*?\"}", Pattern.CASE_INSENSITIVE);

    /**
     * 异步回调地址配置
     */
    @Value("${payConfig.alipay.returnUrl}")
    private String returnUrlConfig;

    /**
     * 同步回调地址配置
     */
    @Value("${payConfig.alipay.notifyUrl}")
    private String notifyUrlConfig;

    /**
     * 异步回调地址
     */
    private String returnUrl;

    /**
     * 同步回调地址
     */
    private String notifyUrl;

    /**
     * 支付配置
     */
    private Config config;

    /**
     * 支付调起
     *
     * @param product : 商品参数
     * @return : 返回调起支付参数
     */
    @Override
    public ResultMap create(ProductPayParamBO product) {
        this.attach = product.getAttach();
        this.configInit(getPayConfig(product.getGameId(), product.getPayWay()));
        //回调地址中拼接游戏具体支付方式
        this.returnUrl = this.returnUrlConfig + "?" + "gamePayWayId=" + attach.getGamePayWayId();
        this.notifyUrl = this.notifyUrlConfig + "?" + "gamePayWayId=" + attach.getGamePayWayId();
        switch (product.getPayDevice()) {
            case 1:
                return this.pc(product);
            case 2:
                return this.h5Pay(product);
            case 3:
                return this.app(product);
            default:
                throw new RuntimeException("未知支付方式");
        }
    }

    /**
     * 异步回调
     *
     * @param request  : 请求参数
     * @param response : 返回参数
     * @return : 返回结果
     */
    @Override
    public String notify(HttpServletRequest request, HttpServletResponse response) {
        try {
            Map<String, String> params = new HashMap<>();
            Enumeration<String> parameterNames = request.getParameterNames();
            while (parameterNames.hasMoreElements()) {
                String parameterName = parameterNames.nextElement();
                params.put(parameterName, request.getParameter(parameterName));
            }
            String orderNo = request.getParameter("out_trade_no");
            String gamePayWayId = params.get("gamePayWayId");
            //回调地址拼接参数不参加sign验证
            params.remove("gamePayWayId");
            //初始化配置
            configInit(getPayConfig(gamePayWayId));
            Boolean flag = Factory.Payment.Common().verifyNotify(params);
            if (flag) {
                if (paySuccess(orderNo, params.get("total_amount"), params.get("trade_no"), gamePayWayId)) {
                    log.info("支付宝异步回调成功 request:{},orderNo:{},商户订单号为：{}", request, orderNo, params.get("trade_no"));
                    return ResEnum.SUCCESS.getMsg();
                }
            }
            log.error("支付宝异步回调-验证签名失败 request:{}", request);
        } catch (Exception e) {
            e.printStackTrace();
        }
        return ResEnum.FAIL.getMsg();
    }

    /**
     * 同步回调
     *
     * @param request : 请求参数
     * @return : 返回结果
     */
    @Override
    public ResultMap synNotify(HttpServletRequest request) {
        try {
            //获取支付宝GET过来反馈信息
            Map<String, String> params = new HashMap<>();
            Map<String, String[]> requestParams = request.getParameterMap();
            for (String name : requestParams.keySet()) {
                String[] values = requestParams.get(name);
                String valueStr = "";
                for (int i = 0; i < values.length; i++) {
                    valueStr = (i == values.length - 1) ? valueStr + values[i]
                            : valueStr + values[i] + ",";
                }
                //乱码解决，这段代码在出现乱码时使用
                valueStr = new String(valueStr.getBytes("ISO-8859-1"), StandardCharsets.UTF_8);
                params.put(name, valueStr);
            }
            //商户订单号
            String orderNo = new String(request.getParameter("out_trade_no").getBytes("ISO-8859-1"), Constants.UTF8);
            String gamePayWayId = params.get("gamePayWayId");
            //回调地址拼接参数不参加sign验证
            params.remove("gamePayWayId");
            //初始化配置
            configInit(getPayConfig(gamePayWayId));
            //验证签名
            Boolean flag = Factory.Payment.Common().verifyNotify(params);
            if (flag) {
                log.info("订单号" + orderNo + "验证签名结果[成功].");
                if (paySuccess(orderNo, params.get("total_amount"), params.get("trade_no"), gamePayWayId)) {
                    log.info("支付宝同步回调成功 request:{},orderNo:{},商户订单号为：{}", request, orderNo, params.get("trade_no"));
                    return ResultMap.ok();
                }
            }
            log.error("订单号" + orderNo + "验证签名结果[失败].");
            return ResultMap.error(HttpStatusEnum.FAIL.getMsg());
        } catch (Exception e) {
            e.printStackTrace();
            //处理异常信息
            return ResultMap.error(e.getMessage());
        }
    }

    /**
     * h5支付
     *
     * @param product : 商品参数
     * @return : 返回支付调起参数
     */
    private ResultMap h5Pay(ProductPayParamBO product) {
        try {
            AlipayTradeWapPayResponse response = Factory.Payment
                    .Wap().asyncNotify(notifyUrl)
                    .pay(product.getSubject(), product.getOutTradeNo(), product.getTotalFee(), "", this.returnUrl);
            //请求失败
            if (!ResponseChecker.success(response)) {
                log.error("支付宝h5支付失败，product : {}, response : {}", JsonUtil.toString(product), JsonUtil.toString(response));
                return ResultMap.error(ResEnum.FAIL.getMsg());
            }
            HashMap<String, String> result = new HashMap<>();
            String body = response.getBody();
            result.put("fromData", body);
            //安卓与ios系统时才单独获取app支付地址
            if (!product.getDeviceSystem().contains(OsEnum.SYSTEM_ANDROID.getOs()) && !product.getDeviceSystem().contains(OsEnum.SYSTEM_IOS.getOs())) {
                return ResultMap.ok(product.getPayDevice(), result, product.getOutTradeNo());
            }
            Matcher matcher = pattern.matcher(body);
            //解析短链
            if (matcher.find()) {
                String url = matcher.group();
                url = url.substring(8, url.length() - 2);
                Matcher paraMatcher = param.matcher(body);
                if (paraMatcher.find()) {
                    String paramData = paraMatcher.group();
                    paramData = paramData.substring(7, paramData.length() - 2);
                    paramData = paramData.replace("&quot;", "\"");
                    String mobileClientUrl = HttpUtil.postDataUrl(url, "biz_content=" + paramData, null);
                    String aliPayContentBody = HttpUtil.postData(mobileClientUrl, "");
                    Matcher mobileClientMatcher = mobileClientParam.matcher(aliPayContentBody);
                    String deepLinkAndroid = "alipays://platformapi/startApp?appId=20000125&orderSuffix=";
                    String deepLinkIos = "alipay://alipayclient/?";
                    if (mobileClientMatcher.find()) {
                        String pageData = mobileClientMatcher.group() + "}";
                        JSONObject jsonObject = JSONObject.parseObject(pageData);
                        Object invokeAliPayData = jsonObject.get("invokeAlipayData");
                        String dataString = JSONObject.parseObject(invokeAliPayData.toString()).get("dataString").toString();
                        deepLinkAndroid += URLEncoder.encode(dataString, Constants.UTF8);
                        deepLinkIos += URLEncoder.encode(pageData, Constants.UTF8);
                        if (product.getDeviceSystem().contains(OsEnum.SYSTEM_ANDROID.getOs())) {
                            result.put("appLink", deepLinkAndroid);
                        } else {
                            result.put("appLink", deepLinkIos);
                        }
                    }
                }
            }
            return ResultMap.ok(product.getPayDevice(), result.get("appLink"), product.getOutTradeNo());
        } catch (Exception e) {
            log.error("支付宝h5支付失败，meg : {}", e.getMessage());
            return ResultMap.error(ResEnum.FAIL.getMsg());
        }
    }

    /**
     * pc端电脑支付
     *
     * @param product : 商品参数
     * @return : 返回支付调起参数
     */
    private ResultMap pc(ProductPayParamBO product) {
        try {
            log.info("pc端支付生成，请求参数 bo:{}", product);
            AlipayTradePagePayResponse response = Factory.Payment
                    .Page().asyncNotify(notifyUrl)
                    .pay(product.getSubject(), product.getOutTradeNo(), product.getTotalFee(), this.returnUrl);
            log.info("pc端支付生成，result:{}", response);
            if (ResponseChecker.success(response)) {
                return ResultMap.ok(product.getPayDevice(), response.getBody(), product.getOutTradeNo());
            } else {
                return ResultMap.error(ResEnum.FAIL.getMsg());
            }
        } catch (Exception e) {
            log.error("pc端支付生成异常 e:{}", e.getMessage());
            return ResultMap.error(ResEnum.FAIL.getMsg());
        }
    }

    /**
     * app支付
     *
     * @param product : 商品参数
     * @return : 返回支付调起参数
     */
    private ResultMap app(ProductPayParamBO product) {
        try {
            AlipayTradeAppPayResponse response = Factory.Payment
                    .App().asyncNotify(notifyUrl)
                    .pay(product.getSubject(), product.getOutTradeNo(), product.getTotalFee());
            if (ResponseChecker.success(response)) {
                return ResultMap.ok(product.getPayDevice(), response.getBody(), product.getOutTradeNo());
            } else {
                return ResultMap.error(ResEnum.FAIL.getMsg());
            }
        } catch (Exception e) {
            e.printStackTrace();
            log.error("app端支付生成异常 e:{}", e.getMessage());
            return ResultMap.error(ResEnum.FAIL.getMsg());
        }
    }

    /**
     * 配置初始化
     *
     * @param obj : 配置参数
     */
    @Override
    public void configInit(String obj) {
        Config config = JSONObject.parseObject(obj, Config.class);
        Factory.setOptions(config);
        this.config = config;
    }
}