package com.zanxiang.sdk.service.pay;

import com.zanxiang.common.domain.ResultMap;
import com.zanxiang.common.enums.OrderStateEnum;
import com.zanxiang.mybatis.entity.GamePayWay;
import com.zanxiang.sdk.domain.bo.PlatformOrderBO;
import com.zanxiang.sdk.domain.bo.ProductPayAttachParamBO;
import com.zanxiang.sdk.domain.bo.ProductPayParamBO;
import com.zanxiang.sdk.domain.dto.PlatformOrderDTO;
import com.zanxiang.sdk.service.GamePayWayService;
import com.zanxiang.sdk.service.PlatformOrderService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Lazy;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.math.BigDecimal;
import java.time.LocalDateTime;

/**
 * @author : lingfeng
 * @time : 2022-09-28
 * @description : 支付超类
 */
@Slf4j
public abstract class PayBaseService {

    @Lazy
    @Autowired
    public PlatformOrderService platformOrderService;

    @Lazy
    @Autowired
    public GamePayWayService gamePayWayService;

    /**
     * 自定义参数
     */
    protected ProductPayAttachParamBO attach;

    /**
     * 创建订单支付参数
     *
     * @param product
     */
    public ResultMap payCreate(ProductPayParamBO product) {
        try {
            //获取订单信息
            PlatformOrderDTO info = platformOrderService.info(product.getOutTradeNo());
            if (info == null) {
                throw new RuntimeException("订单不存在");
            }
            String gameId = String.valueOf(info.getGameId());
            //根据订单中游戏id，获取游戏的支付方式
            GamePayWay payInfo = gamePayWayService.getPayWayToOrderPay(gameId, product.getPayWay());
            product.setProductId(info.getProductId());
            product.setSubject(info.getProductName());
            product.setGameId(gameId);
            product.setTotalFee(String.valueOf(info.getAmount()));
            product.setOutTradeNo(info.getId());
            product.setConfig(payInfo.getPayConfig());
            ProductPayAttachParamBO attachBO = new ProductPayAttachParamBO();
            attachBO.setUserId(product.getUserId());
            attachBO.setOrderId(product.getOutTradeNo());
            attachBO.setPayWay(product.getPayWay());
            attachBO.setGamePayWayId(payInfo.getId());
            product.setAttach(attachBO);
            this.attach = attachBO;
            ResultMap result = this.create(product);
            log.info("订单支付参数生成 ProductPayParamBO:{},result:{}", product, result);
            return result;
        } catch (RuntimeException e) {
            e.printStackTrace();
            log.info("订单支付参数生成异常 ProductPayParamBO:{},e:{}", product, e);
            return ResultMap.error(e.getMessage());
        }
    }

    /**
     * 支付成功逻辑
     *
     * @param orderNo         订单号
     * @param realAmount      真实金额
     * @param merchantOrderNo 订单流水号
     * @param gamePayWayId    具体游戏支付方式id
     * @return : 返回支付状态
     */
    public Boolean paySuccess(String orderNo, String realAmount, String merchantOrderNo, String gamePayWayId) {
        PlatformOrderBO bo = new PlatformOrderBO();
        bo.setId(orderNo);
        bo.setRealAmount(new BigDecimal(realAmount));
        bo.setMerchantOrderNo(merchantOrderNo);
        bo.setStatus(OrderStateEnum.SUCCESS.getCode());
        bo.setGamePaywayId(gamePayWayId);
        bo.setPayTime(LocalDateTime.now());
        return platformOrderService.pay(bo);
    }

    /**
     * 获取支付配置
     *
     * @param gameId 游戏id
     * @param payWay 支付方式
     */
    public String getPayConfig(String gameId, Integer payWay) {
        GamePayWay payInfo = gamePayWayService.getPayWayToOrderPay(gameId, payWay);
        this.attach.setGamePayWayId(payInfo.getId());
        return payInfo.getPayConfig();
    }

    /**
     * 获取支付配置
     *
     * @param gamePayWay 支付方式id
     */
    public String getPayConfig(String gamePayWay) {
        return gamePayWayService.getInfo(gamePayWay).getPayConfig();
    }


    /**
     * 支付调起
     *
     * @param product : 调起支付的参数
     * @return : 返回支付参数
     */
    public abstract ResultMap create(ProductPayParamBO product);

    /**
     * 支付异步回调
     *
     * @param request  : 回调请求
     * @param response : 回调返回
     * @return : 返回回调结果
     * @throws IOException : io异常
     */
    public abstract String notify(HttpServletRequest request, HttpServletResponse response) throws IOException;

    /**
     * 支付同步回调
     *
     * @param request : 回调参数
     * @return : 返回回调结果
     */
    public abstract ResultMap synNotify(HttpServletRequest request);

    /**
     * 配置初始化
     *
     * @param obj : 配置参数
     */
    public abstract void configInit(String obj);
}
