package com.zanxiang.sdk.service.pay;

import com.zanxiang.common.constant.Constants;
import com.zanxiang.common.domain.ResultMap;
import com.zanxiang.common.enums.ResEnum;
import com.zanxiang.common.exception.BaseException;
import com.zanxiang.common.utils.URIUtil;
import com.zanxiang.module.util.JsonUtil;
import com.zanxiang.sdk.constant.WxPayConstants;
import com.zanxiang.sdk.domain.bo.ProductPayAttachParamBO;
import com.zanxiang.sdk.domain.bo.ProductPayParamBO;
import com.zanxiang.sdk.domain.bo.WxPayConfigBO;
import com.zanxiang.sdk.util.HttpUtil;
import com.zanxiang.sdk.util.WxPayUtil;
import com.zanxiang.sdk.util.XmlUtil;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.io.IOUtils;
import org.apache.logging.log4j.util.Strings;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.web.client.RestTemplate;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.BufferedOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.nio.charset.StandardCharsets;
import java.util.HashMap;
import java.util.Map;
import java.util.Objects;

/**
 * @author xufeng
 * @date 2022/6/8 15:37
 */
@Slf4j
@Service
public class WxPayService extends PayBaseService {

    /**
     * pc二维码支付
     */
    private static final String WX_PAY_NATIVE = "NATIVE";

    /**
     * h5支付
     */
    private static final String WX_PAY_MWEB = "MWEB";

    /**
     * 小程序支付
     */
    private static final String WX_PAY_JSAPI = "JSAPI";

    /**
     * 服务器域名
     */
    @Value("${server.domain}")
    private String serverUrl;

    /**
     * 微信回调接口
     */
    @Value("${payConfig.wx.notifyUrl}")
    private String notifyUrl;

    /**
     * 微信支付配置
     */
    private WxPayConfigBO config;

    @Autowired
    private RestTemplate restTemplate;

    @Override
    public ResultMap create(ProductPayParamBO product) {
        this.attach = product.getAttach();
        //初始化配置
        this.configInit(getPayConfig(product.getGameId(), product.getPayWay()));
        //不同的支付途径
        switch (product.getPayDevice()) {
            case 1:
                //PC
                return this.pcPay(product);
            case 2:
                //H5
                return this.h5Pay(product);
            case 4:
                //小程序
                return this.miniAppPay(product);
            default:
                throw new RuntimeException("未知支付方式");
        }
    }

    @Override
    public String notify(HttpServletRequest request, HttpServletResponse response) throws IOException {
        // 读取参数
        InputStream inputStream = request.getInputStream();
        String requestStr = IOUtils.toString(inputStream, StandardCharsets.UTF_8);
        // 解析xml成map
        Map<String, String> packageParams = XmlUtil.xmlToMap(requestStr);
        //获取订单信息
        String attachStr = packageParams.get("attach");
        ProductPayAttachParamBO attachBO = JsonUtil.toObj(attachStr, ProductPayAttachParamBO.class);
        if (attachBO == null) {
            log.info("回调参数中attach值为空");
            return null;
        }
        configInit(getPayConfig(attachBO.getGamePayWayId()));
        // 账号信息
        String key = config.getApiKey();
        // 判断签名是否正确
        String mySign = WxPayUtil.generateSignature(packageParams, key);
        if (!Objects.equals(mySign, packageParams.get("sign"))) {
            log.error("通知签名验证失败, muSign : {}, sign : {}", mySign, packageParams.get("sign"));
            return null;
        }
        log.info("微信支付成功回调");
        // 处理业务开始
        Map<String, String> xmlMap = new HashMap<>();
        if (Constants.SUCCESS.equalsIgnoreCase(packageParams.get("result_code"))) {
            String orderNo = packageParams.get("out_trade_no");
            log.info("微信订单号{}付款成功", orderNo);
            if (paySuccess(attachBO.getOrderId(), String.valueOf(Float.parseFloat(packageParams.get("total_fee")) / 100),
                    packageParams.get("transaction_id"), attachBO.getGamePayWayId())) {
                // 通知微信.异步确认成功.必写.不然会一直通知后台.八次之后就认为交易失败了
                xmlMap.put("return_code", "SUCCESS");
                xmlMap.put("return_msg", "OK");
            }
        } else {
            log.info("支付失败,错误信息：{}", packageParams.get("err_code"));
            xmlMap.put("return_code", "FAIL");
            xmlMap.put("return_msg", "报文为空");
        }
        // 处理业务完毕
        BufferedOutputStream out = new BufferedOutputStream(response.getOutputStream());
        log.info("返回给腾讯的回调结果, ss : {}", XmlUtil.mapToXml(xmlMap));
        out.write(XmlUtil.mapToXml(xmlMap).getBytes());
        out.flush();
        out.close();
        return ResEnum.SUCCESS.getMsg();
    }

    private ResultMap pcPay(ProductPayParamBO product) {
        //下单
        Map<String, String> successMap = this.unifiedOrder(product, WX_PAY_NATIVE, null);
        //获取二维码链接
        String urlCode = successMap.get("code_url");
        //判断是否获取到支付链接
        if (Strings.isBlank(urlCode)) {
            log.error("微信PC支付链接urlCode为空, product：{}", JsonUtil.toString(product));
            throw new BaseException("微信PC支付链接urlCode为空");
        }
        //返回结果
        return ResultMap.ok(product.getPayDevice(), urlCode, product.getOutTradeNo());
    }

    private ResultMap h5Pay(ProductPayParamBO product) {
        //下单
        Map<String, String> successMap = this.unifiedOrder(product, WX_PAY_MWEB, null);
        //获取h5支付链接
        String urlCode = successMap.get("mweb_url");
        if (Strings.isBlank(urlCode)) {
            log.error("微信h5支付链接urlCode为空, product：{}", JsonUtil.toString(product));
            throw new BaseException("微信h5支付链接urlCode为空");
        }
        //转换deepLink
        String deepLink;
        try {
            deepLink = WxPayUtil.readUrl(urlCode, serverUrl);
        } catch (Exception e) {
            log.error("微信h5支付urlCode转deepLink异常, urlCode：{}", urlCode);
            throw new BaseException("微信h5支付urlCode转deepLink异常");
        }
        //返回结果
        return ResultMap.ok(product.getPayDevice(), deepLink, product.getOutTradeNo());
    }

    private ResultMap miniAppPay(ProductPayParamBO product) {
        try {
            Map<Object, Object> miniMap = new HashMap<>(5);
            // 请求微信服务器，使用code获取openid
            Map<String, String> paramMap = new HashMap<>(4);
            paramMap.put("appid", config.getAppId());
            paramMap.put("secret", config.getAppSecret());
            paramMap.put("js_code", product.getCode());
            paramMap.put("grant_type", "authorization_code");
            // 发送请求
            String url = URIUtil.fillUrlParams("https://api.weixin.qq.com/sns/jscode2session", paramMap, Boolean.FALSE);
            String sr = restTemplate.getForObject(url, String.class);
            // 解析相应内容（转换成json对象）
            Map<String, String> userMap = JsonUtil.toMap(sr, Map.class, String.class);
            if (userMap == null || Strings.isBlank(userMap.get("openid"))) {
                throw new BaseException("小程序支付获取用户openId为空");
            }
            //下单
            Map<String, String> successMap = this.unifiedOrder(product, WX_PAY_JSAPI, userMap.get("openid"));
            // 支付参数
            String prepayId = successMap.get("prepay_id");
            // 随机字符串
            String nonceStr = WxPayUtil.generateNonceStr();
            // 时间戳转化成字符串，否则前端wx.requestPayment方法会报签名错误
            String createTime = String.valueOf(System.currentTimeMillis() / 1000);
            // 获取统一下单回调结果, 二次签名
            HashMap<String, String> map = new HashMap<>(5);
            map.put("appId", config.getAppId());
            map.put("timeStamp", createTime);
            map.put("nonceStr", nonceStr);
            map.put("package", "prepay_id=" + prepayId);
            map.put("signType", config.getSignType());
            // 再次签名sign，这个签名用于小程序端调用支付方法
            String sign = WxPayUtil.generateSignature(map, config.getApiKey());
            //组装返回结果
            miniMap.put("paySign", sign);
            miniMap.put("signType", config.getSignType());
            miniMap.put("timeStamp", createTime);
            miniMap.put("nonceStr", nonceStr);
            miniMap.put("package", successMap);
            return ResultMap.ok(miniMap);
        } catch (Exception e) {
            log.error("微信小程序支付通信异常, 订单号：{}, e : {}", product.getOutTradeNo(), e.getMessage());
            return ResultMap.error();
        }
    }

    private Map<String, String> unifiedOrder(ProductPayParamBO product, String tradeType, String openId) {
        try {
            Map<String, String> paramData = new HashMap<>(13);
            paramData.put("appid", config.getAppId());
            paramData.put("mch_id", config.getMchId());
            paramData.put("nonce_str", WxPayUtil.generateNonceStr());
            paramData.put("body", product.getSubject());
            paramData.put("out_trade_no", product.getOutTradeNo());
            paramData.put("fee_type", "CNY");
            paramData.put("total_fee", WxPayUtil.subZeroAndDot(String.valueOf(Float.parseFloat(product.getTotalFee()) * 100)));
            paramData.put("spbill_create_ip", product.getSpbillCreateIp());
            paramData.put("notify_url", notifyUrl);
            paramData.put("trade_type", tradeType);
            paramData.put("attach", JsonUtil.toString(product.getAttach()));
            paramData.put("sign_type", config.getSignType());
            //小程序商城下单需要携带openId
            if (Strings.isNotBlank(openId)) {
                paramData.put("openid", openId);
            }
            //接口签名
            String sign = WxPayUtil.generateSignature(paramData, config.getApiKey());
            paramData.put("sign", sign);
            // 下单, 获取结果
            String result = HttpUtil.postData(WxPayConstants.UNIFIED_ORDER_URL, XmlUtil.mapToXml(paramData));
            Map<String, String> successMap = XmlUtil.xmlToMap(result);
            // 返回状态码
            String returnCode = successMap.get("return_code");
            // 结果状态码
            String resultCode = successMap.get("result_code");
            //成功, 返回结果
            if (Constants.SUCCESS.equalsIgnoreCase(returnCode) && returnCode.equals(resultCode)) {
                return successMap;
            }
            //下单失败
            log.error("微信小程序支付通信异常, paramData：{}", JsonUtil.toString(paramData));
            throw new BaseException("微信小程序支付下单失败");
        } catch (Exception e) {
            log.error("微信小程序支付通信异常, 订单号：{}, e : {}", product.getOutTradeNo(), e.getMessage());
            throw new BaseException("微信小程序支付下单异常");
        }
    }

    /**
     * 配置初始化
     */
    @Override
    public void configInit(String obj) {
        this.config = JsonUtil.toObj(obj, WxPayConfigBO.class);
    }

    @Override
    public ResultMap synNotify(HttpServletRequest request) {
        return null;
    }
}
